/*
Program: cipher-gen
File: cipher-gen.h
Author: Marti Voros
*/

#ifndef CIPHER_GEN_H_
#define CIPHER_GEN_H_

#include<iostream>
#include<string>
#include<sstream>

#define MYVERSION 0.1
#define LINE 1000
using namespace std;

/*Types definitions*/
typedef Z<2> coeff_t;
typedef MPol<coeff_t> mpol_t; // multivariate poly over QQ
typedef MatrDse<mpol_t, lapack::rep2d<mpol_t> > matrix_t; //LFSR
typedef MatrDse<coeff_t, lapack::rep2d<coeff_t> > lkey_t;

typedef struct unit {
	coeff_t constant;
	vector<int> index;
	vector<int> exp;
};

typedef struct interval {
	int a;
	int b;
};

/*Cipher Functions */
void init_cipher();
void init_combinator(char * func);
void make_step();
void eval();
void load_lfsr(char * input);
void set_range(char * s);
void print_state();
void print_trans();
void print_key();
void print_combinator();
void print_logic(ofstream &out, mpol_t &eq);

/*Support*/
string itos(int i);
bool inter_look_up(int i);
void usage ();
void arguments(int argc, char* argv[]);

#endif /*CIPHER_GEN_H_*/
