/*
Program: xl
File: hash.cpp
Author: Martin Voros
*/

#include "hash.h"

/*
 * Hash table functions
 */
void *
xmalloc (size_t size)
{
	register void *value = malloc (size);
	if (value == 0) {
		fprintf(stderr,"Virtual memory exhausted!\n");
		exit(1);
	}
	return value;
}

void 
free_cell(HASH_CELL * p)
{
	if (p->next != NULL)
		free_cell(p->next);
	free(p);
}

unsigned long
hash_mono(mpol_t::iterator mono, unsigned long mod)
{
	unsigned long hash = 5381;
	
	for (int j=0; j<=mono->nvars(); j++) {
		hash = ((hash << 5) + hash) + mono->rep()[j];
	}

        return hash %  mod;
}

HASH_CELL ** 
ht_alloc(unsigned long size)
{
	unsigned long i;
	HASH_CELL ** table=NULL;
	
	table = (HASH_CELL **)xmalloc( size * sizeof(HASH_CELL *));
	for (i=0; i<size; i++) table[i]=NULL;
	
	return table;
}

void
ht_free(HASH_CELL * table[], unsigned long size)
{
	for (unsigned long i=0; i<size; i++) {
		if (table[i] != NULL) {
			free_cell(table[i]);
		}
	}
}

unsigned long
hash_lookup(mpol_t::iterator mono, HASH_CELL * list)
{
	HASH_CELL * p;

	p=list;
	while (p != NULL) {
		if (*(p->it) == *mono)
			return p->id;
		p=p->next;
	}	

	return 0;
}


HASH_CELL * 
insert_mono(mpol_t::iterator it, HASH_CELL * list, vector<mpol_t::iterator> &vec, unsigned long id)
{
	HASH_CELL * p = NULL;

	if (list == NULL) {
		p=(HASH_CELL *)xmalloc(sizeof(HASH_CELL));
		p->id=id;
		p->it=it;
		p->next=NULL;
		vec.push_back(it);
		return p;
	} else {
		p=list;
		while (p->next != NULL) p=p->next;
		p->next=(HASH_CELL *)malloc(sizeof(HASH_CELL));
		p->next->id=id;
		p->next->it=it;
		p->next->next=NULL;
		vec.push_back(it);
		return list; 
	}
}


/*
 * End of Hash table functions
 */
