/*
Program: xl
File: xl.h
Author: Marti Voros
*/

#ifndef XL_H_
#define XL_H_

#include<iostream>
#include<string>
#include<sstream>
#include<vector>

/*SYNAPS headers*/
#include "synaps/mpol.h"
#include "synaps/arithm/Zp.h"
#include "synaps/linalg/lapack.h"
#include "synaps/linalg/MatrDse.h"

#define MYVERSION 0.3
#define FIELD 2

/* Types definitions */
typedef Z<FIELD> coeff_t;
typedef MPol<coeff_t> mpol_t; // multivariate poly over coeff_t
typedef MatrDse<coeff_t, lapack::rep2d<coeff_t> > matrix_t;

using namespace std;

void load_eq(char * input, vector<mpol_t> &E);
void print_eqs(vector<mpol_t> &eqs);
void print_sol(); 
void init_vars(mpol_t & mpol);
void generate_product(vector<mpol_t> &E);
void reduce_equations(vector<mpol_t> &E, int max);
void eval_poly(mpol_t &m,mpol_t &r, vector<int> &sol);
void linearize(vector<mpol_t> &eqs);
long uni_check(mpol_t::iterator mono);
long uni_check2(long row);
int deg(mpol_t::iterator mono);
long mono_member(mpol_t::iterator mono);
unsigned long g2elim(matrix_t &A, vector<coeff_t> &B);
coeff_t eval_mrow(long row, coeff_t val);
coeff_t solve_brutegf2(long row);

/* Support */
void usage ();
void arguments(int argc, char* argv[]);

#endif /*XL_H_*/
